package com.cloudbility.openapi;

import com.cloudbility.openapi.request.ImportHostRequest;
import com.cloudbility.openapi.request.ImportPublicHostRequest;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.CloudHostVo;
import com.cloudbility.openapi.vo.ImportHostVo;
import com.cloudbility.openapi.vo.JobVo;
import com.cloudbility.openapi.vo.ShortcutVo;

import java.util.List;
import java.util.Map;

public interface HostApi {

    HostGroupVoResult getHostGroupByCloud(long cloudId) throws OpenapiException;

    /**
     * 匹配规则：内网IP=innerIp（如果设置） 且 公网IP=publicIp（如果设置）。innerIp和publicIp两个参数必须至少指定一个
     *
     * @param innerIp
     * @param publicIp
     * @return
     */
    CloudHostResult findByInnerIpAndPublicIp(long teamId, Long cloudId, String innerIp, String publicIp) throws OpenapiException;

    CloudHostResult findByInstanceId(long teamId, Long cloudId, String instanceId) throws OpenapiException;

    CloudHostResult getByTeam(long teamId, Integer page, Integer size) throws OpenapiException;

    CloudHostResult getByUser(long teamId, long userId) throws OpenapiException;

    CloudHostResult getByUser(long teamId, long userId, Long cloudId, Long groupId) throws OpenapiException;

    CloudHostResult getByTeamCode(String teamCode, Integer page, Integer size) throws OpenapiException;

    CloudHostResult getByCloud(long cloudId, Integer page, Integer size) throws OpenapiException;


    GetHostBriefResult getHostBriefList(long teamId) throws OpenapiException;

    Map<Long, String> getHostStatusMap(long teamId, List<Long> hostIds) throws OpenapiException;

    CloudHostVo getHost(long hostId) throws OpenapiException;

//    ListCredentialsResult getHostCredentials(long hostId) throws OpenapiException;

    GetAgentScriptResult getAgentScript(long hostId) throws OpenapiException;

    /**
     * 匹配规则：内网IP或公网IP匹配给定IP
     *
     * @param ip
     * @return
     * @throws OpenapiException
     */
    CloudHostResult findByIp(long teamId, Long cloudId, String ip) throws OpenapiException;

    ImportHostResult importHosts(ImportHostRequest request) throws OpenapiException;

    JobVo importPublic(long cloudId, String regionId, String instanceId, Long creatorId) throws OpenapiException;

//    ImportHostResult importHost(long teamId, long cloudId, ImportHostVo hostVo, List<Integer> vgIds) throws OpenapiException;

    JobVo importPublic(ImportPublicHostRequest request) throws OpenapiException;

    void updateHost(long hostId, ImportHostVo hostVo) throws OpenapiException;

    void revokeAllRolesFromHg(Long hgId) throws OpenapiException;

    ListCredentialsResult updateHostCredentials(long hostId, String user, String password) throws OpenapiException;

    void deleteHosts(List<Long> hostIds) throws OpenapiException;

    GetShortcutsResult getHostShortcuts(long hostId) throws OpenapiException;

    int deleteHostShortcuts(long hostId) throws OpenapiException;

    ShortcutVo getShortcut(Long shortcutId) throws OpenapiException;

    void deleteShortcut(Long shortcutId) throws OpenapiException;

    // 成本api
    HostCostResult getHostCostByTeam(long teamId) throws OpenapiException;

    HostCostResult getHostCostByCloud(long cloudId) throws OpenapiException;

    OAuthUrlResult getHostCostCsvUrlByTeam(long teamId, String lang, boolean oneoff, long expireSeconds) throws OpenapiException;

    OAuthUrlResult getHostCostCsvUrlByCloud(long cloudId, String lang, boolean oneoff, long expireSeconds) throws OpenapiException;

    //成本优化建议api
    OptimizedAdviceResult getHostOptimizedAdviceByTeam(long teamId) throws OpenapiException;

    OptimizedAdviceResult getHostOptimizedAdviceByCloud(long cloudId) throws OpenapiException;

    OptimizedAdviceResult getHostOptimizedAdvice(long hostId) throws OpenapiException;

    /**
     * 启动虚拟机
     *
     * @param hostId
     * @throws OpenapiException
     */
    void startHost(long hostId) throws OpenapiException;

    /**
     * 关闭虚拟机
     *
     * @param hostId
     * @throws OpenapiException
     */
    void stopHost(long hostId) throws OpenapiException;

    /**
     * 重启虚拟机
     *
     * @param hostId
     * @param force  是否强制重启
     * @throws OpenapiException
     */
    void restartHost(long hostId, Boolean force) throws OpenapiException;

    //主机访问串api
    AccessAuthResult createHostAccessAuth(Long hostId, Long userId, Long credentialId) throws OpenapiException;

    HostAccessAuthResult getAccessAuthByHost(Long hostId, Long userId) throws OpenapiException;

    AccessAuthResult getAccessAuth(Long accessAuthId) throws OpenapiException;

    void deleteAccessAuthById(Long accessAuthId) throws OpenapiException;

    void deleteAccessAuthByHost(Long hostId, Long userId) throws OpenapiException;

    PublicIpHostsResult findPublicHostsByTeam(long teamId, String createTimeFrom, String createTimeTo) throws OpenapiException;

    /**
     * 查询团队内安装了Agent的主机
     * @param teamId
     * @return
     * @throws OpenapiException
     */
    CloudHostResult findHostsWithAgent(Long teamId)throws OpenapiException;

    /**
     * 查询团队内安装了Proxy的主机
     * @param teamId
     * @return
     * @throws OpenapiException
     */
    CloudHostResult findHostsWithProxy(Long teamId)throws OpenapiException;
}
