package com.cloudbility.openapi;

import com.cloudbility.openapi.vo.*;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.List;

public interface ScriptApi {

    ScriptLibraryListResult getScriptLibraryList(long teamId) throws OpenapiException;

    void updateScriptLibrary(long libraryId, String name) throws OpenapiException;

    ScriptLibraryVo createScriptLibrary(long teamId, String name, String description, Long creator) throws OpenapiException;

    ScriptLibraryVo getScriptLibrary(long id) throws OpenapiException;

    /**
     * @param teamId
     * @param name
     * @return
     * @throws OpenapiException
     * @since 4.13
     */
    ScriptLibraryVo getScriptLibraryByName(long teamId, String name) throws OpenapiException;

    void deleteScriptLibrary(long libraryId) throws OpenapiException;

    int deleteScriptsByLibrary(long libraryId) throws OpenapiException;

    ScriptListResult getScriptList(long libraryId) throws OpenapiException;

    ScriptVo createScript(long libraryId, String name, String content, String parameter, String description, ScriptLanguage language, Integer timeout, Long creator) throws OpenapiException;

    ScriptVo getScript(long id) throws OpenapiException;

    ScriptVo getScript(long id, boolean includeContent) throws OpenapiException;

    ScriptVo getScriptByName(long libraryId, String name, boolean includeContent) throws OpenapiException;

    void deleteScript(long id) throws OpenapiException;

    void updateScriptInfo(long id, String name, int timeout, String description) throws OpenapiException;

    void updateScriptContent(long id, String content) throws OpenapiException;

    void updateScriptContentWithPayload(long id, InputStream inputStream, long length) throws OpenapiException;

    void updateScriptContentWithPayload(long id, File file) throws OpenapiException, FileNotFoundException;

    JobVo executeScript(int scriptId, ScriptExecutingHostsInfo executingHostsInfo) throws OpenapiException;

    ScriptRunResult getScriptRunResult(long jobId, boolean includeOutput) throws OpenapiException;

    /**
     * @param jobId
     * @param hostId
     * @return
     * @throws OpenapiException
     */
    InputStreamResult getScriptOutputOnHost(long jobId, long hostId) throws OpenapiException;

    ScriptLibraryListResult getPreInstalledScriptLibraries(long teamId) throws OpenapiException;

    ScriptLibraryListResult getPreInstalledSubLibrary(long teamId, long libraryId) throws OpenapiException;

    ScriptListResult getScriptsByPreInstalledLibrary(long libraryId, boolean includeContent, int page, int size) throws OpenapiException;

    /**
     * 通过Agent执行脚本
     * @param hostIds  主机ID列表,多个ID用英文逗号分开
     * @param platform 平台,linux|windows
     * @param command  命令
     * @throws OpenapiException
     */
    JobVo executeCommandWithAgent(List<Long> hostIds, String platform, String command) throws OpenapiException;

}
