package com.cloudbility.openapi;

import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.*;

import java.util.Date;
import java.util.List;

public interface UserApi {
    GetUserResult getUserByAccount(String account) throws OpenapiException;

    GetUserResult getUserByPhone(String phone) throws OpenapiException;

    GetUserResult getUserByEmail(String email) throws OpenapiException;

    UserBriefResult getUserBriefList(Long teamId) throws OpenapiException;

    PagedUserListResult getUsersByTeam(Long teamId, int page, int size) throws OpenapiException;

    UserListResult getUserByNickname(String nickname) throws OpenapiException;

    UserListResult getUsersByCreateTime(Date createTimeFrom, Date createTimeTo) throws OpenapiException;

    UserListResult getUsersByCursor(int cursor, int size) throws OpenapiException;

    UserBriefResult getUserBriefListByTeamCode(String teamCode) throws OpenapiException;

    ImportUserResult importUsers(List<ImportUserVo> users) throws OpenapiException;

    //post /userV2
    ImportUserResult importUser(ImportUserVo userVo) throws OpenapiException;

    GetUserResult getUser(long userId) throws OpenapiException;

    PagedUserListResult getGetUserList(Integer page, Integer size) throws OpenapiException;

    GetFunctionResult getUserFunctionsByTeam(Long teamId, Long userId) throws OpenapiException;

    GetFunctionResult getUserFunctionsByJwtAndTeam(Long teamId, String jwt) throws OpenapiException;

    GetFunctionResult getUserFunctionsByTeam(Long teamId, Long userId, String module, FunctionType type) throws OpenapiException;

    GetFunctionResult getUserFunctionsByJwtAndTeam(Long teamId, String jwt, String module, FunctionType type) throws OpenapiException;

    TeamListResult getJoinedTeams(long userId) throws OpenapiException;

    RoleListResult findJoinedRoles(Long teamId, Long userId) throws OpenapiException;

    RoleListResult findJoinedRoles(Long teamId, String account) throws OpenapiException;

    UserListResult getUsersByName(String name) throws OpenapiException;

    UserListResult getUsersByNameLike(String name) throws OpenapiException;

    void updateUserByAccount(String account, UpdateUserVo updateUserVo) throws OpenapiException;

    void updateUserEmailByAccount(String account, String email) throws OpenapiException;

    void updateUserPasswordByAccount(String account, String password) throws OpenapiException;

    void updateUserEmail(long userId, String email) throws OpenapiException;

    void updateUserPassword(long userId, String password) throws OpenapiException;

    void updateUserAccount(long userId, String newAccount) throws OpenapiException;

    void updateUserAccount(String account, String newAccount) throws OpenapiException;

    void blockUser(long userId, int blockingSeconds, String description) throws OpenapiException;

    void blockUser(String account, int blockingSeconds, String description) throws OpenapiException;

    void unblockUser(long userId) throws OpenapiException;

    void unblockUser(String account) throws OpenapiException;

    void deleteUserByAccount(String account) throws OpenapiException;

    void deleteUsers(List<Long> userIds) throws OpenapiException;

    void verifyIdCard(String realName, String idCard) throws OpenapiException;

    int countOnlineUsers() throws OpenapiException;

    int countOnlineMembers(long teamId) throws OpenapiException;

    /**
     * 创建或者更新用户扩展属性
     *
     * @param userId      用户ID
     * @param paramName   属性名
     * @param paramValue  属性值
     * @param description 参数描述信息
     * @return
     * @throws OpenapiException
     */
    WorkTaskVo.UserExtendParamVo saveOrUpdateUserExtendParam(Long userId, String paramName, String paramValue, String description) throws OpenapiException;

    /**
     * 删除用户扩展属性
     *
     * @param userId    用户ID
     * @param paramName 属性名,为空表示删除该用户的所有扩展属性
     * @throws OpenapiException
     */
    int deleteUserExtendParam(Long userId, String paramName) throws OpenapiException;

    /**
     * 查询用户扩展属性
     *
     * @param userId     用户ID
     * @param paramName  属性名
     * @param paramValue 属性值
     * @return
     * @throws OpenapiException
     */
    WorkTaskVo.UserExtendParamResult getUserExtendParams(Long userId, String paramName, String paramValue) throws OpenapiException;

    /**
     * 根据扩展属性查询用户
     *
     * @param paramName  属性名
     * @param paramValue 属性值
     * @return
     * @throws OpenapiException
     */
    UserListResult getUserListByExtendParam(String paramName, String paramValue) throws OpenapiException;

    /**
     * 判断用户是否在团队中
     *
     * @param teamId
     * @param userId
     * @return
     */
    boolean isUserInTeam(Long teamId, Long userId) throws OpenapiException;

    /**
     * 创建用户
     *
     * @param account          登录账号，必填
     * @param nickname         昵称，非必填
     * @param phone            手机号，非必填
     * @param email            邮箱，非必填
     * @param password         密码，为空时自动生成一个密码，非必填
     * @param thirdPartyUserId 第三方用户唯一标识，非必填
     * @param joinTeamCode     需要加入的团队标识，非必填
     * @param roleIds          角色ID列表,以半角逗号分隔
     * @param departmentId     部门ID
     * @return
     * @throws OpenapiException
     */
    SimpleUserVo createUser(String account, String nickname, String phone, String email, String password, String thirdPartyUserId, String joinTeamCode, String roleIds, Long departmentId) throws OpenapiException;

    /**
     * 根据第三方用户ID查询
     * @param thirdPartyUserId
     * @return
     * @throws OpenapiException
     */
    SimpleUserVo queryByThirdPartyUserId(String thirdPartyUserId)throws OpenapiException;
}
