package com.cloudbility.openapi.client;

import com.cloudbility.openapi.KeyPairApi;
import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.response.KeyPairListResult;
import com.cloudbility.openapi.response.OpenapiResponse;
import com.cloudbility.openapi.vo.KeyPairDetailVo;
import com.cloudbility.openapi.vo.KeyPairVo;

public class DefaultKeyPairApi extends AbstractApi implements KeyPairApi {

    public DefaultKeyPairApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public KeyPairListResult listKeyParis(long teamId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/keyPairs")
                .paramShouldNotEmpty("teamId", teamId)
                .build(), KeyPairListResult.class);
    }

    @Override
    public KeyPairDetailVo getKeyPairInfo(Long keyPairId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/keyPair/" + keyPairId)
                .build(), KeyPairDetailVo.class);
    }

    @Override
    public KeyPairVo createKeyPair(long teamId, String name, String description, String key) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .post("/keyPair")
                .paramShouldNotEmpty("teamId", teamId)
                .paramShouldNotEmpty("name", name)
                .paramIfNotNull("description", description)
                .paramIfNotNull("key", key)
                .build(), KeyPairVo.class);
    }

    @Override
    public void updateKeyPairName(Long keyPairId, String name, String description) throws OpenapiException {
        execute(RequestorBuilder.create()
                .put("/keyPair/" + keyPairId)
                .paramShouldNotEmpty("name", name)
                .paramIfNotNull("description", description)
                .build(), OpenapiResponse.class);
    }

    @Override
    public void updateKeyPairMembers(Long keyPairId, String members) throws OpenapiException {
        execute(RequestorBuilder.create()
                .put("/keyPair/" + keyPairId + "/members")
                .paramIfNotNull("members", members)
                .build(), OpenapiResponse.class);
    }

    @Override
    public void updateKeyPairRoles(Long keyPairId, String roles) throws OpenapiException {
        execute(RequestorBuilder.create()
                .put("/keyPair/" + keyPairId + "/roles")
                .paramIfNotNull("roles", roles)
                .build(), OpenapiResponse.class);
    }

    @Override
    public void updateKeyPairDepartments(Long keyPairId, String departments) throws OpenapiException {
        execute(RequestorBuilder.create()
                .put("/keyPair/" + keyPairId + "/departments")
                .paramIfNotNull("departments", departments)
                .build(), OpenapiResponse.class);
    }

    @Override
    public void deleteKeyPair(Long keyPairId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .delete("/keyPair/" + keyPairId)
                .build(), OpenapiResponse.class);
    }
}
