package com.cloudbility.openapi.client;

import com.cloudbility.openapi.MessageApi;
import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.common.Base64Util;
import com.cloudbility.openapi.common.JsonUtil;
import com.cloudbility.openapi.common.StringUtil;
import com.cloudbility.openapi.vo.MessageSendTemplate;

import java.io.File;
import java.util.List;
import java.util.Map;

/**
 * Created by lidonghao on 2020/8/13.
 */
public class DefaultMessageApi extends AbstractApi implements MessageApi {

    public DefaultMessageApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public void sendSmsMessage(Long teamId, MessageSendTemplate template, Map<String, Object> values) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .method(HttpMethod.POST)
                        .path("/message/sms")
                        .paramShouldNotEmpty("teamId", teamId)
                        .paramShouldNotEmpty("template", template)
                        .paramIfNotNull("values", values != null ? JsonUtil.toJson(values) : null)
                        .build(),
                Void.class);
    }

    @Override
    public void sendMailMessage(Long teamId, MessageSendTemplate template, Map<String, Object> values, String subject, File attachment) throws OpenapiException {
        String _values = values != null ? JsonUtil.toJson(values) : null;
        RequestorBuilder builder = RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/message/mail")
                .paramShouldNotEmpty("teamId", teamId)
                .paramShouldNotEmpty("template", template);
        if (attachment != null) {
            builder = builder.enableMultipart()
                    .multipart("attachment", attachment.getName(), attachment)
                    .paramIfNotNull("values", StringUtil.notEmpty(_values) ? Base64Util.encode(_values.getBytes()) : null)//有附件时,如果参数中有中文,就会报错,所以这里先转为Base64
                    .paramIfNotNull("subject", StringUtil.notEmpty(subject) ? Base64Util.encode(subject.getBytes()) : null)
                    .build();
        } else {
            builder = builder.paramIfNotNull("values", _values).paramIfNotNull("subject", subject);
        }
        execute(builder, Void.class);
    }
}
