package com.cloudbility.openapi.client;

import com.cloudbility.openapi.MonitorApi;
import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiErrorCode;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.common.DateUtil;
import com.cloudbility.openapi.common.StringUtil;
import com.cloudbility.openapi.vo.Scope;
import com.cloudbility.openapi.vo.monitor.*;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

import static java.util.stream.Collectors.toList;

public class DefaultMonitorApi extends AbstractApi implements MonitorApi {
    public DefaultMonitorApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public SingletonResultList getHostAggregation(long hostId, MonitorMetric monitorMetric, List<Func> funcList, Date startTime, Date endTime) throws OpenapiException {
        return execute(RequestorBuilder.create().get("/monitor/hostAggregation/" + hostId)
                        .paramShouldNotEmpty("metric", monitorMetric.name())
                        .paramIfNotNull("functions", StringUtil.join(funcList, ","))
                        .paramIfNotNull("startTime", startTime != null ? DateUtil.formatDate(startTime) : null)
                        .paramIfNotNull("endTime", endTime != null ? DateUtil.formatDate(endTime) : null)
                ,
                SingletonResultList.class);
    }

    @Override
    public BatchHostAggregationData getHostsAggregation(List<Long> hostIds, MonitorMetric monitorMetric, List<Func> funcList, Date startTime, Date endTime) throws OpenapiException {
        return execute(RequestorBuilder.create().get("/monitor/hostsAggregation/" + StringUtil.join(hostIds, ","))
                        .paramShouldNotEmpty("metric", monitorMetric.name())
                        .paramIfNotNull("functions", StringUtil.join(funcList, ","))
                        .paramIfNotNull("startTime", startTime != null ? DateUtil.formatDate(startTime) : null)
                        .paramIfNotNull("endTime", endTime != null ? DateUtil.formatDate(endTime) : null)
                ,
                BatchHostAggregationData.class);
    }

    @Override
    public BatchHostAggregationData getHostsAggregationByCloud(long cloudId, MonitorMetric monitorMetric, List<Func> funcList, Date startTime, Date endTime) throws OpenapiException {
        return execute(RequestorBuilder.create().get("/monitor/hostsAggregation/byCloud/" + cloudId)
                        .paramShouldNotEmpty("metric", monitorMetric.name())
                        .paramIfNotNull("functions", StringUtil.join(funcList, ","))
                        .paramIfNotNull("startTime", startTime != null ? DateUtil.formatDate(startTime) : null)
                        .paramIfNotNull("endTime", endTime != null ? DateUtil.formatDate(endTime) : null)
                ,
                BatchHostAggregationData.class);
    }

    @Override
    public BatchHostAggregationData getHostsAggregationByTag(long tagId, MonitorMetric monitorMetric, List<Func> funcList, Date startTime, Date endTime) throws OpenapiException {
        return execute(RequestorBuilder.create().get("/monitor/hostsAggregation/byTag/" + tagId)
                        .paramShouldNotEmpty("metric", monitorMetric.name())
                        .paramIfNotNull("functions", StringUtil.join(funcList, ","))
                        .paramIfNotNull("startTime", startTime != null ? DateUtil.formatDate(startTime) : null)
                        .paramIfNotNull("endTime", endTime != null ? DateUtil.formatDate(endTime) : null)
                ,
                BatchHostAggregationData.class);
    }

    @Override
    public BatchHostAggregationData getHostsAggregationByVg(long vgId, MonitorMetric monitorMetric, List<Func> funcList, Date startTime, Date endTime) throws OpenapiException {
        return execute(RequestorBuilder.create().get("/monitor/hostsAggregation/byVg/" + vgId)
                        .paramShouldNotEmpty("metric", monitorMetric.name())
                        .paramIfNotNull("functions", StringUtil.join(funcList, ","))
                        .paramIfNotNull("startTime", startTime != null ? DateUtil.formatDate(startTime) : null)
                        .paramIfNotNull("endTime", endTime != null ? DateUtil.formatDate(endTime) : null)
                ,
                BatchHostAggregationData.class);
    }

    @Override
    public BatchHostAggregationData getHostsAggregationByHg(long hgId, MonitorMetric monitorMetric, List<Func> funcList, Date startTime, Date endTime) throws OpenapiException {
        return execute(RequestorBuilder.create().get("/monitor/hostsAggregation/byHg/" + hgId)
                        .paramShouldNotEmpty("metric", monitorMetric.name())
                        .paramIfNotNull("functions", StringUtil.join(funcList, ","))
                        .paramIfNotNull("startTime", startTime != null ? DateUtil.formatDate(startTime) : null)
                        .paramIfNotNull("endTime", endTime != null ? DateUtil.formatDate(endTime) : null)
                ,
                BatchHostAggregationData.class);
    }

    ////

    @Override
    public AggregationData getHostsAvgAggregation(List<Long> hostIds, MonitorMetric monitorMetric, List<Func> funcList, Date startTime, Date endTime) throws OpenapiException {
        return execute(RequestorBuilder.create().get("/monitor/hostsAvgAggregation/" + StringUtil.join(hostIds, ","))
                        .paramShouldNotEmpty("metric", monitorMetric.name())
                        .paramIfNotNull("functions", StringUtil.join(funcList, ","))
                        .paramIfNotNull("startTime", startTime != null ? DateUtil.formatDate(startTime) : null)
                        .paramIfNotNull("endTime", endTime != null ? DateUtil.formatDate(endTime) : null)
                ,
                AggregationData.class);
    }

    @Override
    public AggregationData getHostsAvgAggregationByCloud(long cloudId, MonitorMetric monitorMetric, List<Func> funcList, Date startTime, Date endTime) throws OpenapiException {
        return execute(RequestorBuilder.create().get("/monitor/hostsAvgAggregation/byCloud/" + cloudId)
                        .paramShouldNotEmpty("metric", monitorMetric.name())
                        .paramIfNotNull("functions", StringUtil.join(funcList, ","))
                        .paramIfNotNull("startTime", startTime != null ? DateUtil.formatDate(startTime) : null)
                        .paramIfNotNull("endTime", endTime != null ? DateUtil.formatDate(endTime) : null)
                ,
                AggregationData.class);
    }

    @Override
    public AggregationData getHostsAvgAggregationByTag(long tagId, MonitorMetric monitorMetric, List<Func> funcList, Date startTime, Date endTime) throws OpenapiException {
        return execute(RequestorBuilder.create().get("/monitor/hostsAvgAggregation/byTag/" + tagId)
                        .paramShouldNotEmpty("metric", monitorMetric.name())
                        .paramIfNotNull("functions", StringUtil.join(funcList, ","))
                        .paramIfNotNull("startTime", startTime != null ? DateUtil.formatDate(startTime) : null)
                        .paramIfNotNull("endTime", endTime != null ? DateUtil.formatDate(endTime) : null)
                ,
                AggregationData.class);
    }

    @Override
    public AggregationData getHostsAvgAggregationByVg(long vgId, MonitorMetric monitorMetric, List<Func> funcList, Date startTime, Date endTime) throws OpenapiException {
        return execute(RequestorBuilder.create().get("/monitor/hostsAvgAggregation/byVg/" + vgId)
                        .paramShouldNotEmpty("metric", monitorMetric.name())
                        .paramIfNotNull("functions", StringUtil.join(funcList, ","))
                        .paramIfNotNull("startTime", startTime != null ? DateUtil.formatDate(startTime) : null)
                        .paramIfNotNull("endTime", endTime != null ? DateUtil.formatDate(endTime) : null)
                ,
                AggregationData.class);
    }

    @Override
    public AggregationData getHostsAvgAggregationByHg(long hgId, MonitorMetric monitorMetric, List<Func> funcList, Date startTime, Date endTime) throws OpenapiException {
        return execute(RequestorBuilder.create().get("/monitor/hostsAvgAggregation/byHg/" + hgId)
                        .paramShouldNotEmpty("metric", monitorMetric.name())
                        .paramIfNotNull("functions", StringUtil.join(funcList, ","))
                        .paramIfNotNull("startTime", startTime != null ? DateUtil.formatDate(startTime) : null)
                        .paramIfNotNull("endTime", endTime != null ? DateUtil.formatDate(endTime) : null)
                ,
                AggregationData.class);
    }

    @Override
    public MetricSingletonMap getHostLastMetricData(long hostId, MonitorMetric... monitorMetric) throws OpenapiException {
        if (monitorMetric == null || monitorMetric.length == 0) {
            throw new OpenapiException(OpenapiErrorCode.InvalidParameter("monitorMetric not set"));
        }
        String metrics = StringUtil.join(Arrays.asList(monitorMetric).stream().map(m -> m.name()).collect(toList()), ",");
        return execute(RequestorBuilder.create().get("/monitor/hostLast/" + hostId)
                        .paramShouldNotEmpty("metrics", metrics),
                MetricSingletonMap.class);
    }

    @Override
    public MetricSingletonMap getZStackHostLastMetricData(Scope scope, long hostId, MonitorMetric... monitorMetric) throws OpenapiException {
        if (monitorMetric == null || monitorMetric.length == 0) {
            throw new OpenapiException(OpenapiErrorCode.InvalidParameter("monitorMetric not set"));
        }
        String metrics = StringUtil.join(Arrays.asList(monitorMetric).stream().map(m -> m.name()).collect(toList()), ",");
        return execute(RequestorBuilder.create().get("/monitor/zstackHostLast/" + hostId)
                        .paramShouldNotEmpty("scope", scope)
                        .paramShouldNotEmpty("metrics", metrics),
                MetricSingletonMap.class);
    }
}
