package com.cloudbility.openapi.common.networking;

import com.cloudbility.openapi.client.OpenapiConfig;
import org.apache.http.client.HttpClient;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.HttpExecutionAware;
import org.apache.http.client.methods.HttpRequestWrapper;
import org.apache.http.client.protocol.HttpClientContext;
import org.apache.http.config.Registry;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.config.SocketConfig;
import org.apache.http.conn.HttpClientConnectionManager;
import org.apache.http.conn.routing.HttpRoute;
import org.apache.http.conn.socket.ConnectionSocketFactory;
import org.apache.http.conn.socket.PlainConnectionSocketFactory;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.impl.execchain.ClientExecChain;
import org.springframework.web.util.UriComponentsBuilder;

import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import java.net.URI;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

public class HttpClientBuilder {
    public final static SSLContext NOP_TLSV12_SSL_CONTEXT;

    public static final X509TrustManager NOP_TRUST_MANAGER = new X509TrustManager() {
        @Override
        public void checkClientTrusted(X509Certificate[] x509Certificates, String s)
                throws CertificateException {
        }

        @Override
        public void checkServerTrusted(X509Certificate[] x509Certificates, String s)
                throws CertificateException {
        }

        @Override
        public X509Certificate[] getAcceptedIssuers() {
            return new X509Certificate[0];
        }
    };

    static {
        try {
            NOP_TLSV12_SSL_CONTEXT = SSLContext.getInstance("TLSv1.2");//禁用了TLSV1.0
            NOP_TLSV12_SSL_CONTEXT.init(null, new TrustManager[]{NOP_TRUST_MANAGER}, new java.security.SecureRandom());
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        } catch (KeyManagementException e) {
            throw new RuntimeException(e);
        }
    }

    public static SSLContext getSSLContext() {
        return NOP_TLSV12_SSL_CONTEXT;
    }

    public static HttpClient buildHttpClient(OpenapiConfig openapiConfig) {
        HttpClientConnectionManager connectionManager = createHttpClientConnectionManager();
        return new EncodePlusHttpClientBuilder().setConnectionManager(connectionManager)
                .setUserAgent(openapiConfig.getUserAgent() + "-" + openapiConfig.getVersion())
                .disableContentCompression()
                .disableAutomaticRetries()
                .setDefaultRequestConfig(RequestConfig.custom()
                        .setConnectTimeout(openapiConfig.getConnectTimeout())
                        .setSocketTimeout(openapiConfig.getReadTimeout()).build())
                .build();
    }

    static class EncodePlusHttpClientBuilder extends org.apache.http.impl.client.HttpClientBuilder {
        @Override
        protected ClientExecChain decorateMainExec(final ClientExecChain requestExecutor) {
            return (
                    HttpRoute route,
                    HttpRequestWrapper request,
                    HttpClientContext clientContext,
                    HttpExecutionAware execAware) -> {
                URI u = request.getURI();
                if (u.getRawQuery() != null) {
                    String strictlyEscapedQuery = u.getRawQuery().replace("+", "%2B");
                    u = UriComponentsBuilder.fromUri(u)
                            .replaceQuery(strictlyEscapedQuery)
                            .build(true).toUri();
                }
                request.setURI(u);
                return requestExecutor.execute(route, request, clientContext, execAware);
            };
        }
    }

    private static HttpClientConnectionManager createHttpClientConnectionManager() {
        Registry<ConnectionSocketFactory> socketFactoryRegistry = RegistryBuilder.<ConnectionSocketFactory>create()
                .register("http", new PlainConnectionSocketFactory())
                .register("https", new SSLConnectionSocketFactory(NOP_TLSV12_SSL_CONTEXT, NoopHostnameVerifier.INSTANCE))
                .build();
        PoolingHttpClientConnectionManager connectionManager = new PoolingHttpClientConnectionManager(socketFactoryRegistry);
        connectionManager.setDefaultMaxPerRoute(2);
        connectionManager.setMaxTotal(10);
        connectionManager.setValidateAfterInactivity(2 * 1000);
        connectionManager.setDefaultSocketConfig(SocketConfig.custom().
                setSoTimeout(50 * 1000).setTcpNoDelay(true).build());
        return connectionManager;
    }
}
