package com.cloudbility.openapi.demo.batch2;

import com.cloudbility.common.util.CSVUtil;
import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.client.OpenapiConfig;
import com.cloudbility.openapi.common.DemoUtil;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Properties;

/**
 *
 */
public class BatchImporterMain2 {

    public static OpenapiClient createOpenapiClient(String accessKeyId, String accessKeySecret, String endpoint) {
        OpenapiConfig config = new OpenapiConfig();
        config.setAccessKeyId(accessKeyId);
        config.setAccessKeySecret(accessKeySecret);
        config.setEndpoint(endpoint);
        return new OpenapiClient(config);
    }

    public static void main(String[] args) throws IOException {
        if (args.length == 0) {
            System.err.println("no batch_config.properties");
            return;
        }
        String configFile = args[0];
        Properties prop = DemoUtil.load(configFile);
        System.out.println("loaded config from " + configFile);
        String accessKeyId = prop.getProperty("accessKeyId");
        String accessKeySecret = prop.getProperty("accessKeySecret");
        String endpoint = prop.getProperty("endpoint");
        String cvsFile = prop.getProperty("cvsFile");
        Long teamId = Long.parseLong(prop.getProperty("teamId"));
        Long cloudId = Long.parseLong(prop.getProperty("cloudId"));
        if (teamId <= 0) {
            System.err.println("please set teamId to config file");
            return;
        }
        if (cloudId <= 0) {
            System.err.println("please set cloudId to config file");
            return;
        }
        String cvsFileCharset = prop.getProperty("cvsFileCharset", "UTF-8");
        importStart(accessKeyId, accessKeySecret, endpoint, cvsFile, teamId, cloudId, cvsFileCharset);
    }

    public static void importStart(String accessKeyId, String accessKeySecret, String endpoint, String cvsFile, long teamId, long cloudId, String cvsFileCharset) throws FileNotFoundException {
        OpenapiClient openapiClient = createOpenapiClient(accessKeyId, accessKeySecret, endpoint);
        BatchImporter2 importer = new BatchImporter2(openapiClient);
        try {
            List<String[]> rows = CSVUtil.readCsvFileWithCommaField(new FileInputStream(new File(cvsFile)), cvsFileCharset);
            List<HostVgVo2> hosts = new ArrayList<>();
            for (int i = 1; i < rows.size(); i++) {
                String[] row = rows.get(i);
                HostVgVo2 hv = new HostVgVo2();
                hv.setInstanceName(row[0]);
                hv.setIp(row[1]);
                hv.setPlatform(row[2]);
                hv.setVgL1(row[3]);
                hv.setVgL2(row[4]);
                hv.setVgL3(row[5]);
                hv.setAccounts(Arrays.asList(row[6].split("，|,")));
                hosts.add(hv);
            }
            importer.importHostAndUser(teamId, cloudId, hosts);
        } catch (IOException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }

    }
}
