package com.cloudbility.openapi.impl;

import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.common.SelfCancelable;
import com.cloudbility.openapi.ScriptListener;
import com.cloudbility.openapi.vo.JobVo;
import com.cloudbility.openapi.vo.ScriptRunResult;

public abstract class AbstractPoolingJob extends SelfCancelable implements Runnable, ScriptListener {
    private JobVo jobVo;
    private OpenapiClient openapiClient;

    public AbstractPoolingJob(OpenapiClient openapiClient, JobVo jobVo) {
        this.jobVo = jobVo;
        this.openapiClient = openapiClient;
    }

    @Override
    public void run() {
        String prevName = Thread.currentThread().getName();
        try {
            Thread.currentThread().setName(prevName + "=>Script-" + jobVo.getId());
            ScriptRunResult result = openapiClient.scriptApi().getScriptRunResult(jobVo.getId(), false);
            if (result.isDone()) {
                onDone(jobVo, result);
                cancel();//正常结束，取消当前定时任务
            }
        } catch (Throwable throwable) {
            onError(jobVo, throwable);
            cancel();//异常结束，取消当前定时任务
        } finally {
            Thread.currentThread().setName(prevName);
        }
    }

    @Override
    public abstract void onDone(JobVo jobVo, ScriptRunResult result);

    @Override
    public abstract void onError(JobVo jobVo, Throwable throwable);
}