package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.response.ResourceResult;
import com.cloudbility.openapi.vo.AuthorizationInfo;
import com.cloudbility.openapi.vo.ResourceTypeVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import static org.springframework.util.MimeTypeUtils.APPLICATION_JSON_VALUE;

@Api(tags = "Authorization api", description = "授权API")
public interface AuthorizationApi extends Openapi {

    @ApiOperation(value = "获取某个资源的授权信息")
    @RequestMapping(value = "/resource/{type}/{id}/authorization", method = RequestMethod.GET)
    AuthorizationInfo getAuthorization(
            @ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") long teamId,
            @ApiParam(value = "资源类型", required = true) @PathVariable("type") ResourceTypeVo typeVo,
            @ApiParam(value = "资源ID", required = true) @PathVariable("id") Long id) throws OpenapiException;

    @ApiOperation(value = "更新某个资源的授权信息", notes = "注意，这个接口不是追加授权，如果不设置roleIds、departmentIds、memberIds，将分别取消所有角色、部门、团队成员的授权")
    @RequestMapping(value = "/resource/{type}/{id}/authorization", method = RequestMethod.PUT)
    void updateAuthorization(
            @ApiParam(value = "资源类型", required = true) @PathVariable("type") ResourceTypeVo typeVo,
            @ApiParam(value = "资源ID", required = true) @PathVariable("id") Long id,
            @ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") long teamId,
            @ApiParam(value = "角色ID，用逗号连接") @RequestParam(value = "roleIds", required = false) String roleIdStr,
            @ApiParam(value = "部门ID，用逗号连接") @RequestParam(value = "departmentIds", required = false) String departmentStr,
            @ApiParam(value = "成员ID，用逗号连接") @RequestParam(value = "memberIds", required = false) String memberIdStr) throws OpenapiException;

    @ApiOperation(value = "追加某个资源的授权信息", notes = "注意，这个接口是追加授权")
    @RequestMapping(value = "/resource/{type}/{id}/authorization", method = RequestMethod.POST)
    void grantAuthorization(
            @ApiParam(value = "资源类型", required = true) @PathVariable("type") ResourceTypeVo typeVo,
            @ApiParam(value = "资源ID", required = true) @PathVariable("id") Long id,
            @ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") long teamId,
            @ApiParam(value = "角色ID，用逗号连接") @RequestParam(value = "roleIds", required = false) String roleIdStr,
            @ApiParam(value = "部门ID，用逗号连接") @RequestParam(value = "departmentIds", required = false) String departmentStr,
            @ApiParam(value = "成员ID，用逗号连接") @RequestParam(value = "memberIds", required = false) String memberIdStr) throws OpenapiException;

    @ApiOperation(value = "撤销某个资源的授权信息")
    @RequestMapping(value = "/resource/{type}/{id}/authorization", method = RequestMethod.DELETE)
    void revokeAuthorization(
            @ApiParam(value = "资源类型", required = true) @PathVariable("type") ResourceTypeVo typeVo,
            @ApiParam(value = "资源ID", required = true) @PathVariable("id") Long id,
            @ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") long teamId,
            @ApiParam(value = "角色ID，用逗号连接") @RequestParam(value = "roleIds", required = false) String roleIdStr,
            @ApiParam(value = "部门ID，用逗号连接") @RequestParam(value = "departmentIds", required = false) String departmentStr,
            @ApiParam(value = "成员ID，用逗号连接") @RequestParam(value = "memberIds", required = false) String memberIdStr) throws OpenapiException;

    @ApiOperation(value = "获取用户有权限访问的所有资源")
    @RequestMapping(value = "/resources/{type}/byUser/{userId}", method = RequestMethod.GET)
    ResourceResult getResourcesByUserAuthorization(
            @ApiParam(value = "资源类型", required = true) @PathVariable("type") ResourceTypeVo typeVo,
            @ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") long teamId,
            @ApiParam(value = "用户ID", required = true) @PathVariable(value = "userId") long userId) throws OpenapiException;


}
