package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.response.DepartmentTreeVo;
import com.cloudbility.openapi.response.DepartmentVo;
import com.cloudbility.openapi.response.FlatDepartmentInfo;
import com.cloudbility.openapi.vo.BooleanVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

@Api(tags = "Department api", description = "部门管理API")
public interface DepartmentApi extends Openapi {

    @ApiOperation(value = "查询团队是否开启了组织架构视图", notes = "不可逆：切换为组织架构视图后将无法回退为扁平化用户视图。返回根部门。如果已经开启组织架构视图了，会抛出异常。")
    @RequestMapping(value = "/department/init", method = RequestMethod.GET)
    BooleanVo isDepartmentEnabled(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId) throws OpenapiException;


    @ApiOperation(value = "切换组织架构视图", notes = "不可逆：切换为组织架构视图后将无法回退为扁平化用户视图。返回根部门。如果已经开启组织架构视图了，会抛出异常。")
    @RequestMapping(value = "/department/init", method = RequestMethod.POST)
    DepartmentTreeVo initDepartment(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
                                    @ApiParam(value = "部门名称", required = true) @RequestParam("name") String name,
                                    @ApiParam(value = "部门描述") @RequestParam(required = false, value = "description") String description) throws OpenapiException;

    @ApiOperation(value = "获取部门结构树")
    @RequestMapping(value = "/department", method = RequestMethod.GET)
    DepartmentTreeVo getDepartments(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
                                    @ApiParam(value = "是否包含成员", required = true) @RequestParam(required = false, value = "includeMembers", defaultValue = "true") boolean includeMembers) throws OpenapiException;

    @ApiOperation(value = "获取部门信息（不含下级部门，含成员）")
    @RequestMapping(value = "/department/{departmentId}", method = RequestMethod.GET)
    DepartmentVo getDepartment(@ApiParam(value = "部门ID", required = true) @PathVariable("departmentId") Long departmentId) throws OpenapiException;

    @ApiOperation(value = "获取部门列表（扁平数据）")
    @RequestMapping(value = "/departmentFlat", method = RequestMethod.GET)
    FlatDepartmentInfo getFlatDepartments(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
                                          @ApiParam(value = "是否包含成员", required = true) @RequestParam(required = false, value = "includeMembers", defaultValue = "true") boolean includeMembers) throws OpenapiException;

    @ApiOperation(value = "添加部门")
    @RequestMapping(value = "/department", method = RequestMethod.POST)
    DepartmentVo addDepartment(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
                               @ApiParam(value = "部门名称", required = true) @RequestParam("name") String name,
                               @ApiParam(value = "部门描述") @RequestParam(required = false, value = "description") String description,
                               @ApiParam(value = "父部门ID", required = true) @RequestParam("parentId") Long parentId) throws OpenapiException;

    @ApiOperation(value = "更新部门", notes = "修改名称和描述")
    @RequestMapping(value = "/department/{departmentId}", method = RequestMethod.PUT)
    void updateDepartment(@ApiParam(value = "部门ID", required = true) @PathVariable("departmentId") Long departmentId,
                          @ApiParam(value = "部门名称", required = true) @RequestParam("name") String name,
                          @ApiParam(value = "部门描述") @RequestParam(required = false, value = "description") String description) throws OpenapiException;

    @ApiOperation(value = "删除部门", notes = "删除之前保证部门下没有团队成员和子部门")
    @RequestMapping(value = "/department/{departmentId}", method = RequestMethod.DELETE)
    void deleteDepartment(@ApiParam(value = "部门ID", required = true) @PathVariable("departmentId") Long departmentId) throws OpenapiException;

    @ApiOperation(value = "移动成员到部门", notes = "成员会从原部门移除。返回当前部门信息")
    @RequestMapping(value = "/department/{departmentId}/members", method = RequestMethod.PUT)
    void moveToDepartment(@ApiParam(value = "部门ID", required = true) @PathVariable("departmentId") Long departmentId,
                          @ApiParam(value = "成员ID列表,用逗号连接", required = true) @RequestParam(value = "memberIds") String memberIds
    ) throws OpenapiException;

}
