package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.enums.zstack.ImageStatusType;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.response.CloudImageListResult;
import com.cloudbility.openapi.response.OpenapiResponse;
import com.cloudbility.openapi.vo.ImagePermissionType;
import com.cloudbility.openapi.vo.Scope;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

@Api(tags = "27 image api", description = "镜像操作API")
public interface ImageApi {
    @ApiOperation(value = "共享镜像")
    @RequestMapping(value = "/image/share", method = RequestMethod.POST)
    ResponseEntity<OpenapiResponse> shareImage(
            @ApiParam(value = "供应商", required = true) @RequestParam("vendor") String vendor,
            @ApiParam(value = "共享者的镜像所在区域ID", required = true) @RequestParam("regionId") String regionId,
            @ApiParam(value = "共享者的镜像ID", required = true) @RequestParam("imageId") String imageId,
            @ApiParam(value = "被共享者的云环境ID", required = true) @RequestParam("cloudId") Long cloudId,
            @ApiParam(value = "被共享者的账号ID", required = false) @RequestParam(value = "accountId", required = false) String accountId,
            @ApiParam(value = "被共享者的项目ID，华为云需要指定项目", required = false) @RequestParam(value = "projectId", required = false) String projectId
    ) throws OpenapiException;

    @ApiOperation(value = "取消共享镜像")
    @RequestMapping(value = "/image/cancelShare", method = RequestMethod.POST)
    ResponseEntity<OpenapiResponse> cancelShareImage(
            @ApiParam(value = "供应商", required = true) @RequestParam("vendor") String vendor,
            @ApiParam(value = "共享者的镜像所在区域ID", required = true) @RequestParam("regionId") String regionId,
            @ApiParam(value = "共享者的镜像ID", required = true) @RequestParam("imageId") String imageId,
            @ApiParam(value = "被共享者的云环境ID", required = true) @RequestParam("cloudId") Long cloudId,
            @ApiParam(value = "被共享者的账号ID", required = false) @RequestParam(value = "accountId", required = false) String accountId,
            @ApiParam(value = "被共享者的项目ID，华为云需要指定项目", required = false) @RequestParam(value = "projectId", required = false) String projectId
    ) throws OpenapiException;

    @ApiOperation(value = "查询zstack镜像列表", response = CloudImageListResult.class)
    @RequestMapping(value = "/image/cloud/{cloudId}/zstackList", method = RequestMethod.GET)
    CloudImageListResult listZStackImages(@ApiParam(value = "云账户ID", required = true) @PathVariable(value = "cloudId") Long cloudId,
                                          @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                          @ApiParam(value = "镜像Id", required = false) @RequestParam(value = "imageId", required = false) String imageId,
                                          @ApiParam(value = "镜像状态", required = false) @RequestParam(value = "status", required = false) ImageStatusType status) throws OpenapiException;

    @ApiOperation(value = "修改镜像权限")
    @RequestMapping(value = "/image/modifyImageSharePermission", method = RequestMethod.POST)
    ResponseEntity<OpenapiResponse> modifyImageSharePermission(
            @ApiParam(value = "供应商", required = true) @RequestParam("vendor") String vendor,
            @ApiParam(value = "云厂商", required = true) @RequestParam("provider") String provider,
            @ApiParam(value = "共享者的镜像所在区域ID", required = true) @RequestParam("regionId") String regionId,
            @ApiParam(value = "共享者的镜像ID", required = true) @RequestParam("imageId") String imageId,
            @ApiParam(value = "被共享者的账号ID", required = false) @RequestParam(value = "accountId", required = false) String accountId,
            @ApiParam(value = "被共享者的项目ID，华为云需要指定项目", required = false) @RequestParam(value = "projectId", required = false) String projectId,
            @ApiParam(value = "类型,SHARE|CANCEL", required = true) @RequestParam(value = "type", required = true, defaultValue = "SHARE") ImagePermissionType type
    ) throws OpenapiException;
}
