package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.vo.Scope;
import com.cloudbility.openapi.vo.monitor.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

@Api(tags = "12 monitor api", description = "监控API")
public interface MonitorApi extends Openapi {

    @ApiOperation(value = "获取主机监控数据在时间段内统计值", response = SingletonResultList.class, notes = "如果不指定开始时间，相当于取监控数据存储最早的时间；如果不指定结束时间，相当于取最后一个监控数据的的时间。" +
            "注意：必须使用聚合函数，即函数的结果是一个数据点。")
    @RequestMapping(value = "/monitor/hostAggregation/{hostId}", method = RequestMethod.GET)
    SingletonResultList getHostAggregation(
            @ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId,
            @ApiParam(value = "主机监控项", required = true) @RequestParam("metric") MonitorMetric monitorMetric,
            @ApiParam(value = "集合函数列表，多个函数用逗号连接", required = true) @RequestParam("functions") String functions,
            @ApiParam(value = "开始时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "startTime", required = false) String startTimeStr,
            @ApiParam(value = "结束时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "endTime", required = false) String endTimeStr) throws OpenapiException;


    @ApiOperation(value = "获取多台主机监控数据在时间段内统计值", response = BatchHostAggregationData.class, notes = "如果不指定开始时间，相当于取监控数据存储最早的时间；如果不指定结束时间，相当于取最后一个监控数据的的时间。" +
            "注意：必须使用聚合函数，即函数的结果是一个数据点。")
    @RequestMapping(value = "/monitor/hostsAggregation/{hostIds}", method = RequestMethod.GET)
    BatchHostAggregationData getHostsAggregation(
            @ApiParam(value = "主机ID列表，用逗号分隔", required = true) @PathVariable("hostIds") String hostIdsStr,
            @ApiParam(value = "主机监控项", required = true) @RequestParam("metric") MonitorMetric monitorMetric,
            @ApiParam(value = "集合函数列表，多个函数用逗号连接", required = true) @RequestParam("functions") String functions,
            @ApiParam(value = "开始时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "startTime", required = false) String startTimeStr,
            @ApiParam(value = "结束时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "endTime", required = false) String endTimeStr) throws OpenapiException;

    @ApiOperation(value = "获取云账户下所有主机监控数据在时间段内统计值", response = BatchHostAggregationData.class, notes = "如果不指定开始时间，相当于取监控数据存储最早的时间；如果不指定结束时间，相当于取最后一个监控数据的的时间。" +
            "注意：必须使用聚合函数，即函数的结果是一个数据点。")
    @RequestMapping(value = "/monitor/hostsAggregation/byCloud/{cloudId}", method = RequestMethod.GET)
    BatchHostAggregationData getHostsAggregationByCloud(
            @ApiParam(value = "云账户ID", required = true) @PathVariable("cloudId") Long cloudId,
            @ApiParam(value = "主机监控项", required = true) @RequestParam("metric") MonitorMetric monitorMetric,
            @ApiParam(value = "集合函数列表，多个函数用逗号连接", required = true) @RequestParam("functions") String functions,
            @ApiParam(value = "开始时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "startTime", required = false) String startTimeStr,
            @ApiParam(value = "结束时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "endTime", required = false) String endTimeStr) throws OpenapiException;

    @ApiOperation(value = "获取标签获取主机监控数据在时间段内统计值", response = BatchHostAggregationData.class, notes = "如果不指定开始时间，相当于取监控数据存储最早的时间；如果不指定结束时间，相当于取最后一个监控数据的的时间。" +
            "注意：必须使用聚合函数，即函数的结果是一个数据点。")
    @RequestMapping(value = "/monitor/hostsAggregation/byTag/{tagId}", method = RequestMethod.GET)
    BatchHostAggregationData getHostsAggregationByTag(
            @ApiParam(value = "标签ID", required = true) @PathVariable("tagId") Long tagId,
            @ApiParam(value = "主机监控项", required = true) @RequestParam("metric") MonitorMetric monitorMetric,
            @ApiParam(value = "集合函数列表，多个函数用逗号连接", required = true) @RequestParam("functions") String functions,
            @ApiParam(value = "开始时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "startTime", required = false) String startTimeStr,
            @ApiParam(value = "结束时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "endTime", required = false) String endTimeStr) throws OpenapiException;

    @ApiOperation(value = "获取虚拟分组获取主机监控数据在时间段内统计值", response = BatchHostAggregationData.class, notes = "如果不指定开始时间，相当于取监控数据存储最早的时间；如果不指定结束时间，相当于取最后一个监控数据的的时间。" +
            "注意：必须使用聚合函数，即函数的结果是一个数据点。")
    @RequestMapping(value = "/monitor/hostsAggregation/byVg/{vgId}", method = RequestMethod.GET)
    BatchHostAggregationData getHostsAggregationByVg(
            @ApiParam(value = "虚拟分组ID", required = true) @PathVariable("vgId") Long vgId,
            @ApiParam(value = "主机监控项", required = true) @RequestParam("metric") MonitorMetric monitorMetric,
            @ApiParam(value = "集合函数列表，多个函数用逗号连接", required = true) @RequestParam("functions") String functions,
            @ApiParam(value = "开始时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "startTime", required = false) String startTimeStr,
            @ApiParam(value = "结束时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "endTime", required = false) String endTimeStr) throws OpenapiException;


    @ApiOperation(value = "获取物理网络分组获取主机监控数据在时间段内统计值", response = BatchHostAggregationData.class, notes = "如果不指定开始时间，相当于取监控数据存储最早的时间；如果不指定结束时间，相当于取最后一个监控数据的的时间。" +
            "注意：必须使用聚合函数，即函数的结果是一个数据点。")
    @RequestMapping(value = "/monitor/hostsAggregation/byHg/{hgId}", method = RequestMethod.GET)
    BatchHostAggregationData getHostsAggregationByHg(
            @ApiParam(value = "物理网络分组ID", required = true) @PathVariable("hgId") Long hgId,
            @ApiParam(value = "主机监控项", required = true) @RequestParam("metric") MonitorMetric monitorMetric,
            @ApiParam(value = "集合函数列表，多个函数用逗号连接", required = true) @RequestParam("functions") String functions,
            @ApiParam(value = "开始时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "startTime", required = false) String startTimeStr,
            @ApiParam(value = "结束时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "endTime", required = false) String endTimeStr) throws OpenapiException;

    //

    @ApiOperation(value = "获取多台主机监控数据在时间段内统计值的平均值", response = AggregationData.class, notes = "如果不指定开始时间，相当于取监控数据存储最早的时间；如果不指定结束时间，相当于取最后一个监控数据的的时间。" +
            "注意：必须使用聚合函数，即函数的结果是一个数据点。")
    @RequestMapping(value = "/monitor/hostsAvgAggregation/{hostIds}", method = RequestMethod.GET)
    AggregationData getHostsAvgAggregation(
            @ApiParam(value = "主机ID列表，用逗号分隔", required = true) @PathVariable("hostIds") String hostIdsStr,
            @ApiParam(value = "主机监控项", required = true) @RequestParam("metric") MonitorMetric monitorMetric,
            @ApiParam(value = "集合函数列表，多个函数用逗号连接", required = true) @RequestParam("functions") String functions,
            @ApiParam(value = "开始时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "startTime", required = false) String startTimeStr,
            @ApiParam(value = "结束时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "endTime", required = false) String endTimeStr) throws OpenapiException;

    @ApiOperation(value = "获取云账户下所有主机监控数据在时间段内统计值的平均值", response = AggregationData.class, notes = "如果不指定开始时间，相当于取监控数据存储最早的时间；如果不指定结束时间，相当于取最后一个监控数据的的时间。" +
            "注意：必须使用聚合函数，即函数的结果是一个数据点。")
    @RequestMapping(value = "/monitor/hostsAvgAggregation/byCloud/{cloudId}", method = RequestMethod.GET)
    AggregationData getHostsAvgAggregationByCloud(
            @ApiParam(value = "云账户ID", required = true) @PathVariable("cloudId") Long cloudId,
            @ApiParam(value = "主机监控项", required = true) @RequestParam("metric") MonitorMetric monitorMetric,
            @ApiParam(value = "集合函数列表，多个函数用逗号连接", required = true) @RequestParam("functions") String functions,
            @ApiParam(value = "开始时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "startTime", required = false) String startTimeStr,
            @ApiParam(value = "结束时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "endTime", required = false) String endTimeStr) throws OpenapiException;


    @ApiOperation(value = "获取标签下主机监控数据在时间段内统计值的平均值", response = AggregationData.class, notes = "如果不指定开始时间，相当于取监控数据存储最早的时间；如果不指定结束时间，相当于取最后一个监控数据的的时间。" +
            "注意：必须使用聚合函数，即函数的结果是一个数据点。")
    @RequestMapping(value = "/monitor/hostsAvgAggregation/byTag/{tagId}", method = RequestMethod.GET)
    AggregationData getHostsAvgAggregationByTag(
            @ApiParam(value = "标签ID", required = true) @PathVariable("tagId") Long tagId,
            @ApiParam(value = "主机监控项", required = true) @RequestParam("metric") MonitorMetric monitorMetric,
            @ApiParam(value = "集合函数列表，多个函数用逗号连接", required = true) @RequestParam("functions") String functions,
            @ApiParam(value = "开始时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "startTime", required = false) String startTimeStr,
            @ApiParam(value = "结束时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "endTime", required = false) String endTimeStr) throws OpenapiException;

    @ApiOperation(value = "获取虚拟分组中主机监控数据在时间段内统计值的平均值", response = AggregationData.class, notes = "如果不指定开始时间，相当于取监控数据存储最早的时间；如果不指定结束时间，相当于取最后一个监控数据的的时间。" +
            "注意：必须使用聚合函数，即函数的结果是一个数据点。")
    @RequestMapping(value = "/monitor/hostsAvgAggregation/byVg/{vgId}", method = RequestMethod.GET)
    AggregationData getHostsAvgAggregationByVg(
            @ApiParam(value = "虚拟分组ID", required = true) @PathVariable("vgId") Long vgId,
            @ApiParam(value = "主机监控项", required = true) @RequestParam("metric") MonitorMetric monitorMetric,
            @ApiParam(value = "集合函数列表，多个函数用逗号连接", required = true) @RequestParam("functions") String functions,
            @ApiParam(value = "开始时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "startTime", required = false) String startTimeStr,
            @ApiParam(value = "结束时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "endTime", required = false) String endTimeStr) throws OpenapiException;


    @ApiOperation(value = "获取物理网络分组下主机监控数据在时间段内统计值的平均值", response = AggregationData.class, notes = "如果不指定开始时间，相当于取监控数据存储最早的时间；如果不指定结束时间，相当于取最后一个监控数据的的时间。" +
            "注意：必须使用聚合函数，即函数的结果是一个数据点。")
    @RequestMapping(value = "/monitor/hostsAvgAggregation/byHg/{hgId}", method = RequestMethod.GET)
    AggregationData getHostsAvgAggregationByHg(
            @ApiParam(value = "物理网络分组ID", required = true) @PathVariable("hgId") Long hgId,
            @ApiParam(value = "主机监控项", required = true) @RequestParam("metric") MonitorMetric monitorMetric,
            @ApiParam(value = "集合函数列表，多个函数用逗号连接", required = true) @RequestParam("functions") String functions,
            @ApiParam(value = "开始时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "startTime", required = false) String startTimeStr,
            @ApiParam(value = "结束时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "endTime", required = false) String endTimeStr) throws OpenapiException;

    @ApiOperation(value = "获取主机特定监控项的最后一个数据点", response = AggregationData.class, notes = "如果监控项包括不同的dimension，返回的数据会包含所有dimension")
    @RequestMapping(value = "/monitor/hostLast/{hostId}", method = RequestMethod.GET)
    MetricSingletonMap getHostLastMetricData(
            @ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId,
            @ApiParam(value = "主机监控项(MonitorMetric)，用逗号连接", required = true) @RequestParam("metrics") String metrics) throws OpenapiException;

    @ApiOperation(value = "获取ZStack主机特定监控项的最后一个数据点", response = AggregationData.class, notes = "如果监控项包括不同的dimension，返回的数据会包含所有dimension")
    @RequestMapping(value = "/monitor/zstackHostLast/{hostId}", method = RequestMethod.GET)
    MetricSingletonMap getZStackHostLastMetricData(
            @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
            @ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId,
            @ApiParam(value = "主机监控项(MonitorMetric)，用逗号连接", required = true) @RequestParam("metrics") String metrics) throws OpenapiException;

}
