package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.response.OpenapiResponse;
import com.cloudbility.openapi.vo.VirtualGroupDetailVo;
import com.cloudbility.openapi.vo.VirtualGroupResult;
import com.cloudbility.openapi.vo.VirtualGroupVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

@Api(tags = "8 virtual group api", description = "虚拟分组API")
public interface VgApi extends Openapi {
    @ApiOperation(value = "新增虚拟分组", response = VirtualGroupVo.class, notes = "参数roles用逗号连接多个角色id，如果是系统角色，可以用RoleCode。例如：roles='12,TEAM_ADMINER'表示将创建的虚拟分组授权给id=12的角色和团队管理员角色。" +
            "如果参数不设置，则新建的虚拟分组不会添加到任何角色。" +
            "注意：角色的id必须与云账户属于同一个团队。roles不能设置为TEAM_CREATOR。私有部署版包括通过role_templates.json配置的系统角色。默认团队成员授权，支持用户id或者用户账户，用逗号连接")
    @RequestMapping(value = "/vg", method = RequestMethod.POST)
    VirtualGroupVo createVg(@ApiParam(value = "父级分组ID，未设置时创建顶级分组", required = false) @RequestParam(value = "parentVgId", required = false) Long parentVgId,
                                            @ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") Long teamId,
                                            @ApiParam(value = "分组名称", required = true) @RequestParam("name") String name,
                                            @ApiParam(value = "默认角色授权") @RequestParam(value = "roles", required = false) String roles,
                                            @ApiParam(value = "默认团队成员授权") @RequestParam(value = "members", required = false) String members
    ) throws OpenapiException;

    @Deprecated
    @ApiOperation(value = "修改虚拟分组(不推荐）", notes = "包括两部分：修改虚拟分组名称、重新指定授权角色。name和roles两个参数不能同时为空。为空时，不会修改。roles不能设置为TEAM_CREATOR。私有部署版包括通过role_templates.json配置的系统角色")
    @RequestMapping(value = "/vg/{id}", method = RequestMethod.PUT)
    void updateVg(@ApiParam(value = "虚拟分组id", required = true) @PathVariable("id") Long id,
                                             @ApiParam(value = "虚拟分组名称") @RequestParam(value = "name", required = false) String name,
                                             @ApiParam(value = "新的角色授权") @RequestParam(value = "roles", required = false) String roles,
                                             @ApiParam(value = "默认团队成员授权") @RequestParam(value = "members", required = false) String members) throws OpenapiException;

    @ApiOperation(value = "修改虚拟分组名称", notes = "name不能重复")
    @RequestMapping(value = "/vg/{id}/name", method = RequestMethod.PUT)
    void updateVgName(@ApiParam(value = "虚拟分组id", required = true) @PathVariable("id") Long id,
                                 @ApiParam(value = "虚拟分组名称") @RequestParam(value = "name") String name) throws OpenapiException;

    @ApiOperation(value = "授权分组给角色", notes = "roles不能设置为TEAM_CREATOR。私有部署版包括通过role_templates.json配置的系统角色")
    @RequestMapping(value = "/vg/{id}/roles", method = RequestMethod.PUT)
    void updateVgRoles(@ApiParam(value = "虚拟分组id", required = true) @PathVariable("id") Long id,
                                                  @ApiParam(value = "新的角色授权，使用空串时删除所有角色授权", required = true) @RequestParam(value = "roles") String roles) throws OpenapiException;

    @ApiOperation(value = "授权分组给组织单元", notes = "团队必须先开启组织视图")
    @RequestMapping(value = "/vg/{id}/departments", method = RequestMethod.PUT)
    void updateVgDepartments(@ApiParam(value = "虚拟分组id", required = true) @PathVariable("id") Long id,
                                                        @ApiParam(value = "新的组织单元ID列表，用逗号连接，使用空串时删除所有组织单元授权", required = true) @RequestParam(value = "departments") String departments) throws OpenapiException;

    @ApiOperation(value = "授权分组给团队成员")
    @RequestMapping(value = "/vg/{id}/members", method = RequestMethod.PUT)
    void updateVgMembers(@ApiParam(value = "虚拟分组id", required = true) @PathVariable("id") Long id,
                                                    @ApiParam(value = "默认团队成员授权。使用空串时删除所有用户授权", required = true) @RequestParam(value = "members") String members) throws OpenapiException;

    @ApiOperation(value = "撤销虚拟分组的所有角色授权", notes = "清空角色授权以后，只有团队拥有者有权限")
    @RequestMapping(value = "/vg/{id}/revokeAllRoles", method = RequestMethod.PUT)
    void revokeAllVgRoles(@ApiParam(value = "虚拟分组id", required = true) @PathVariable("id") Long id) throws OpenapiException;

    @ApiOperation(value = "删除虚拟分组")
    @RequestMapping(value = "/vg/{ids}", method = RequestMethod.DELETE)
    void delete(@ApiParam(value = "虚拟分组id列表，用','连接", required = true) @PathVariable("ids") String idStr) throws OpenapiException;

    @ApiOperation(value = "添加主机到虚拟分组", notes = "虚拟分组中如果已经包含主机，不会报错")
    @RequestMapping(value = "/vg/{id}/hosts/{hostIds}", method = RequestMethod.POST)
    void addHostsToVg(@ApiParam(value = "虚拟分组id", required = true) @PathVariable("id") Long id,
                                                 @ApiParam(value = "主机id列表，用逗号连接") @PathVariable(value = "hostIds") String hostIdsStr) throws OpenapiException;

    @ApiOperation(value = "从虚拟分组中移除主机", notes = "虚拟分组中如果不包含主机，不会报错")
    @RequestMapping(value = "/vg/{id}/hosts/{hostIds}", method = RequestMethod.DELETE)
    void deleteHostsFromVg(@ApiParam(value = "虚拟分组id", required = true) @PathVariable("id") Long id,
                                                      @ApiParam(value = "主机id列表，用逗号连接", required = true) @PathVariable(value = "hostIds") String hostIdsStr) throws OpenapiException;

    @ApiOperation(value = "从虚拟分组中移除所有主机")
    @RequestMapping(value = "/vg/{id}/hosts", method = RequestMethod.DELETE)
    void deleteHostsFromVg(@ApiParam(value = "虚拟分组id", required = true) @PathVariable("id") Long id) throws OpenapiException;

    @ApiOperation(value = "添加对象存储(Bucket)到虚拟分组", notes = "虚拟分组中如果已经包含对象存储，不会报错")
    @RequestMapping(value = "/vg/{id}/buckets/{bucketIds}", method = RequestMethod.POST)
    void addBucketsToVg(@ApiParam(value = "虚拟分组id", required = true) @PathVariable("id") Long id,
                                                   @ApiParam(value = "对象存储id列表，用逗号连接", required = true) @PathVariable(value = "bucketIds") String bucketIdsStr) throws OpenapiException;

    @ApiOperation(value = "从虚拟分组中移除对象存储")
    @RequestMapping(value = "/vg/{id}/buckets/{bucketIds}", method = RequestMethod.DELETE)
    void deleteBucketsFromVg(@ApiParam(value = "虚拟分组id", required = true) @PathVariable("id") Long id,
                                                        @ApiParam(value = "对象存储id列表，用逗号连接", required = true) @PathVariable(value = "bucketIds") String bucketIdsStr) throws OpenapiException;

    @ApiOperation(value = "从虚拟分组中移除所有对象存储")
    @RequestMapping(value = "/vg/{id}/buckets", method = RequestMethod.DELETE)
    void deleteBucketsFromVg(@ApiParam(value = "虚拟分组id", required = true) @PathVariable("id") Long id) throws OpenapiException;

    @ApiOperation(value = "添加CDN到虚拟分组")
    @RequestMapping(value = "/vg/{id}/cdns/{cdnIds}", method = RequestMethod.POST)
    void addCdnsToVg(@ApiParam(value = "虚拟分组id", required = true) @PathVariable("id") Long id,
                                                @ApiParam(value = "CDN id列表，用逗号连接", required = true) @PathVariable(value = "cdnIds") String cdnIdsStr) throws OpenapiException;

    @ApiOperation(value = "从虚拟分组中移除CDN", notes = "CDN中如果不包含对象，不会报错")
    @RequestMapping(value = "/vg/{id}/cdns/{cdnIds}", method = RequestMethod.DELETE)
    void deleteCdnsFromVg(@ApiParam(value = "虚拟分组id", required = true) @PathVariable("id") Long id,
                                                     @ApiParam(value = "CDN id列表，用逗号连接", required = true) @PathVariable(value = "cdnIds") String cdnIdsStr) throws OpenapiException;

    @ApiOperation(value = "从虚拟分组中移除所有CDN")
    @RequestMapping(value = "/vg/{id}/cdns", method = RequestMethod.DELETE)
    void deleteCdnsFromVg(@ApiParam(value = "虚拟分组id", required = true) @PathVariable("id") Long id) throws OpenapiException;

    @ApiOperation(value = "从虚拟分组中移除所有主机、bucket、CDN资源")
    @RequestMapping(value = "/vg/{id}/allResources", method = RequestMethod.DELETE)
    void deleteResourcesFromVg(@ApiParam(value = "虚拟分组id", required = true) @PathVariable("id") Long id) throws OpenapiException;

    @ApiOperation(value = "获取虚拟分组详情")
    @RequestMapping(value = "/vg/{id}", method = RequestMethod.GET)
    VirtualGroupDetailVo getVg(@ApiParam(value = "虚拟分组id", required = true) @PathVariable("id") Long id) throws OpenapiException;

    @ApiOperation(value = "根据虚拟分组名称查询", notes = "按照团队查询特定名称的虚拟分组")
    @RequestMapping(value = "/vg/byName", method = RequestMethod.GET)
    VirtualGroupResult getVgsByName(
            @ApiParam(value = "团队ID，如果设置，按照团队过滤") @RequestParam(value = "teamId", required = true) Long teamId,
            @ApiParam(value = "虚拟分组名称，精确匹配", required = true) @RequestParam("name") String name) throws OpenapiException;

    @ApiOperation(value = "获取团队虚拟分组")
    @RequestMapping(value = "/vg/byTeam/{teamId}", method = RequestMethod.GET)
    VirtualGroupResult getVgByTeam(@ApiParam(value = "团队ID", required = true) @PathVariable("teamId") Long teamId,
                                                   @ApiParam(value = "父级分组ID，未设置时查询所有分组", required = false) @RequestParam(value = "parentVgId", required = false) Long parentVgId) throws OpenapiException;

    @ApiOperation(value = "获取团队虚拟分组（根据团队code）")
    @RequestMapping(value = "/vg/byTeamCode/{teamCode}", method = RequestMethod.GET)
    VirtualGroupResult getVgByTeam(@ApiParam(value = "团队ID", required = true) @PathVariable("teamCode") String teamCode,
                                                   @ApiParam(value = "父级分组ID，未设置时查询所有分组", required = false) @RequestParam(value = "parentVgId", required = false) Long parentVgId) throws OpenapiException;
}
