package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.response.WorkTaskVoResult;
import com.cloudbility.openapi.response.WorkflowInstanceVoResult;
import com.cloudbility.openapi.vo.WorkItemStageVo;
import com.cloudbility.openapi.vo.WorkTypeVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

@Api(tags = "17 workflow api", description = "工单API")
public interface WorkflowApi extends Openapi {
    @ApiOperation(value = "查询我的流程工单")
    @RequestMapping(value = "/workflow/myInstances", method = RequestMethod.GET)
    WorkflowInstanceVoResult getMyWorkflowInstances(
            @ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
            @ApiParam(value = "当前用户ID", required = true) @RequestParam("userId") Long userId,
            @ApiParam(value = "参与类型：我发起的、我的待办、我的已办", required = true) @RequestParam("stage") WorkItemStageVo stage,
            @ApiParam(value = "按流程类型过滤", required = false) @RequestParam(value = "workType", required = false) WorkTypeVo workType,
            @ApiParam("页码,从1开始") @RequestParam(value = "page", required = false, defaultValue = "1") int page,
            @ApiParam("每页记录数，默认20") @RequestParam(value = "size", required = false, defaultValue = "20") int size,
            @ApiParam("排序字段：createTime、status") @RequestParam(value = "sortField", required = false, defaultValue = "createTime") String sortField,
            @ApiParam("顺序：desc,asc") @RequestParam(value = "sortOrder", required = false, defaultValue = "desc") String sortOrder) throws OpenapiException;

    @ApiOperation(value = "查询流程工作项（按ID）", notes = "按照创建时间升序，查询团队的工作项。游标ID是工作项的ID，查询数据满足id>游标ID")
    @RequestMapping(value = "/workTasks/byCursor", method = RequestMethod.GET)
    WorkTaskVoResult getWorkflowTasksByCursor(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
                                              @ApiParam(value = "按流程类型过滤", required = false) @RequestParam(value = "workType", required = false) WorkTypeVo workType,
                                              @ApiParam(value = "参与类型：我发起的、我的待办、我的已办", required = false) @RequestParam(value = "stage", required = false) WorkItemStageVo stage,
                                              @ApiParam(value = "游标ID", required = true) @RequestParam(value = "cursorId", defaultValue = "0") long cursorId,
                                              @ApiParam("请求记录数，默认20") @RequestParam(value = "size", required = false, defaultValue = "20") int size) throws OpenapiException;

    @ApiOperation(value = "查询流程工作项（按创建时间）", notes = "按照创建时间的起始和截止，查询团队的工作项。按照[fromTime,toTime)的边界进行查询过滤")
    @RequestMapping(value = "/workTasks/byCreateTime", method = RequestMethod.GET)
    WorkTaskVoResult getWorkflowTasksByCreateTime(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
                                                  @ApiParam(value = "按流程类型过滤", required = false) @RequestParam(value = "workType", required = false) WorkTypeVo workType,
                                                  @ApiParam(value = "参与类型：我发起的、我的待办、我的已办", required = false) @RequestParam(value = "stage", required = false) WorkItemStageVo stage,
                                                  @ApiParam(value = "创建时间的起始时间。UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"", required = true) @RequestParam(value = "fromTime") String startTimeStr,
                                                  @ApiParam(value = "创建时间的结束时间。UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"", required = true) @RequestParam(value = "toTime") String endTimeStr
    ) throws OpenapiException;

    @ApiOperation(value = "查询流程工作项（按创建时间）", notes = "按照更新时间的起始和截止，查询团队的工作项。按照[fromTime,toTime)的边界进行查询过滤")
    @RequestMapping(value = "/workTasks/byUpdateTime", method = RequestMethod.GET)
    WorkTaskVoResult getWorkflowTasksByUpdateTime(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
                                                  @ApiParam(value = "按流程类型过滤", required = false) @RequestParam(value = "workType", required = false) WorkTypeVo workType,
                                                  @ApiParam(value = "参与类型：我发起的、我的待办、我的已办", required = false) @RequestParam(value = "stage", required = false) WorkItemStageVo stage,
                                                  @ApiParam(value = "创建时间的起始时间。UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"", required = true) @RequestParam(value = "fromTime") String startTimeStr,
                                                  @ApiParam(value = "创建时间的结束时间。UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"", required = true) @RequestParam(value = "toTime") String endTimeStr
    ) throws OpenapiException;
}
