package com.cloudbility.openapi.vo;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

/**
 * 导入的用户信息
 */
@ApiModel
public class ImportUserVo {
    /**
     * 用户昵称，必须要
     */
    @ApiModelProperty(required = true, value = "用户昵称，必填。")
    private String name;
    /**
     * 用户自定义描述，用户签名，非必须
     */
    @ApiModelProperty(value = "用户签名，可选。用户自定义描述")
    private String introduce;
    /**
     * 性别：0:未知；1,：男；2：女
     */
    @ApiModelProperty(required = true, value = "性别，可选。0:未知；1：男；2：女。不填时默认值为0。", allowableValues = "range[0,2]")
    private int gender = 0;

    /**
     * 地址，非必须
     */
    @ApiModelProperty(value = "地址，可选。")
    private String address;

    @ApiModelProperty(value = "首次登录强制修改密码，可选。默认为false。4.8新增属性。")
    private boolean forceUpdate = false;

    @ApiModelProperty( value = "登录账户，可选。需保证全局唯一,未设置时，将由系统自动生成。4.8新增属性，只能在私有部署版中设置，SAAS版的Openapi不支持设置登录账户。账号不能包含逗号','。")
    private String account;

    @ApiModelProperty(value = "登录密码，可选。长度4~40个字符，即使设置了用户密码策略也不校验复杂性。4.8新增属性。")
    private String password;

    @ApiModelProperty(value = "默认加入的团队，可选。团队ID。4.8新增属性。")
    private Long joinTeam = null;

    @ApiModelProperty(value = "默认加入的团队，可选。团队code，与joinTeam不要同时设置，如果设置了joinTeamCode，会忽略joinTeam。4.10新加字段。")
    private String joinTeamCode = null;

    @ApiModelProperty(value = "在加入团队中默认授权的角色，可选。必须设置joinTeam或者joinTeamIdOrCode才可以设置roles。格式：用逗号','连接的角色ID或者code，code可以设定私有部署环境roles_template.json定义的系统角色。")
    private String roles = null;

    @ApiModelProperty(value = "用户在团队中的有效期，可选。如果设置，会在到期后自动退出团队，如果不设置，则是永久成员。日期格式：yyyy-MM-dd。4.8新增属性。")
    private String expirationDate = null;

    @ApiModelProperty(value = "登录邮箱，可选。需保证全局唯一。4.9新增属性，只能在私有部署版中设置，SAAS用户的邮箱需要通过验证码绑定。")
    private String email = null;

    @ApiModelProperty(value = "是否发送通知邮件，可选。默认为false，只有设置了email才会生效。4.11新增属性。")
    private boolean sendEmail = false;


    public boolean isForceUpdate() {
        return forceUpdate;
    }

    public void setForceUpdate(boolean forceUpdate) {
        this.forceUpdate = forceUpdate;
    }

    public Long getJoinTeam() {
        return joinTeam;
    }

    public void setJoinTeam(Long joinTeam) {
        this.joinTeam = joinTeam;
    }

    public String getRoles() {
        return roles;
    }

    public void setRoles(String roles) {
        this.roles = roles;
    }

    public String getExpirationDate() {
        return expirationDate;
    }

    public void setExpirationDate(String expirationDate) {
        this.expirationDate = expirationDate;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getAccount() {
        return account;
    }

    public void setAccount(String account) {
        this.account = account;
    }

    public String getIntroduce() {
        return introduce;
    }

    public void setIntroduce(String introduce) {
        this.introduce = introduce;
    }

    public int getGender() {
        return gender;
    }

    public void setGender(int gender) {
        this.gender = gender;
    }

    public String getAddress() {
        return address;
    }

    public void setAddress(String address) {
        this.address = address;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public String getJoinTeamCode() {
        return joinTeamCode;
    }

    public void setJoinTeamCode(String joinTeamCode) {
        this.joinTeamCode = joinTeamCode;
    }

    public boolean isSendEmail() {
        return sendEmail;
    }

    public void setSendEmail(boolean sendEmail) {
        this.sendEmail = sendEmail;
    }
}
